import React from 'react';
import ReactDom from 'react-dom';
import { PropTypes } from 'prop-types';
import debounce from 'lodash.debounce';
import throttle from 'lodash.throttle';
import isIntersectionObserverAvailable from '../utils/intersection-observer';
import getScrollAncestor from '../utils/get-scroll-ancestor';

const getScrollX = externalWindow =>
	typeof externalWindow === 'undefined'
		? 0
		: externalWindow.scrollX || externalWindow.pageXOffset;
const getScrollY = externalWindow =>
	typeof externalWindow === 'undefined'
		? 0
		: externalWindow.scrollY || externalWindow.pageYOffset;

const trackWindowScroll = (BaseComponent) => {
	class ScrollAwareComponent extends React.Component {
		constructor(props) {
			super(props);

			this.useIntersectionObserver =
				props.useIntersectionObserver &&
				isIntersectionObserverAvailable();
			if (this.useIntersectionObserver) {
				return;
			}

			const onChangeScroll = this.onChangeScroll.bind(this);

			if (props.delayMethod === 'debounce') {
				this.delayedScroll = debounce(onChangeScroll, props.delayTime);
			} else if (props.delayMethod === 'throttle') {
				this.delayedScroll = throttle(onChangeScroll, props.delayTime);
			}

			this.state = {
				scrollPosition: {
					x: getScrollX(props.externalWindow),
					y: getScrollY(props.externalWindow),
				},
			};

			this.baseComponentRef = React.createRef();
		}

		componentDidMount() {
			this.addListeners();
		}

		componentWillUnmount() {
			this.removeListeners();
		}

		componentDidUpdate() {
			const { externalWindow } = this.props;

			if (
				typeof externalWindow === 'undefined' ||
				this.useIntersectionObserver
			) {
				return;
			}

			const scrollElement = getScrollAncestor(
				ReactDom.findDOMNode(this.baseComponentRef.current)
			);

			if (scrollElement !== this.scrollElement) {
				this.removeListeners();
				this.addListeners();
			}
		}

		addListeners() {
			const { externalWindow } = this.props;

			if (
				typeof externalWindow === 'undefined' ||
				this.useIntersectionObserver
			) {
				return;
			}

			this.scrollElement = getScrollAncestor(
				ReactDom.findDOMNode(this.baseComponentRef.current)
			);

			this.scrollElement.addEventListener('scroll', this.delayedScroll, {
				passive: true,
			});
			externalWindow.addEventListener('resize', this.delayedScroll, {
				passive: true,
			});

			if (this.scrollElement !== externalWindow) {
				externalWindow.addEventListener('scroll', this.delayedScroll, {
					passive: true,
				});
			}
		}

		removeListeners() {
			const { externalWindow } = this.props;

			if (
				typeof externalWindow === 'undefined' ||
				this.useIntersectionObserver
			) {
				return;
			}

			this.scrollElement.removeEventListener(
				'scroll',
				this.delayedScroll
			);
			externalWindow.removeEventListener('resize', this.delayedScroll);

			if (this.scrollElement !== externalWindow) {
				externalWindow.removeEventListener(
					'scroll',
					this.delayedScroll
				);
			}
		}

		onChangeScroll() {
			const { externalWindow } = this.props;

			if (this.useIntersectionObserver) {
				return;
			}

			this.setState({
				scrollPosition: {
					x: getScrollX(externalWindow),
					y: getScrollY(externalWindow),
				},
			});
		}

		render() {
			const { delayMethod, delayTime, ...props } = this.props;
			const scrollPosition = this.useIntersectionObserver
				? null
				: this.state.scrollPosition;

			return (
				<BaseComponent
					forwardRef={this.baseComponentRef}
					scrollPosition={scrollPosition}
					{...props}
				/>
			);
		}
	}

	ScrollAwareComponent.propTypes = {
		delayMethod: PropTypes.oneOf(['debounce', 'throttle']),
		delayTime: PropTypes.number,
		useIntersectionObserver: PropTypes.bool,
	};

	ScrollAwareComponent.defaultProps = {
		delayMethod: 'throttle',
		delayTime: 300,
		useIntersectionObserver: true,
	};

	return ScrollAwareComponent;
};

export default trackWindowScroll;
